DIGIT            [0-9]
EXTENDED_DIGIT   [0-9a-fA-F]
INTEGER          ({DIGIT}(_?{DIGIT})*)
EXPONENT         ([eE](\+?|-){INTEGER})
DECIMAL_LITERAL  {INTEGER}(\.?{INTEGER})?{EXPONENT}?
BASE             {INTEGER}
BASED_INTEGER    {EXTENDED_DIGIT}(_?{EXTENDED_DIGIT})*
BASED_LITERAL    {BASE}#{BASED_INTEGER}(\.{BASED_INTEGER})?#{EXPONENT}?
IDENTIFIER       [a-zA-Z]("_"?[a-zA-Z0-9])*       
STRING_LITERAL   \"([^"]*(\"\")*)*\" 
CHAR_LITERAL     \'[^\n]\'          

----------------------------------------------------------------------------
--                  written for use with aflex
----------------------------------------------------------------------------
--/*------------------------------------------------------------------------*/
--/* Lexical input for AFLEX for LALR(1) Grammar for ANSI Ada83             */
--/*                                                                        */
--/*               Dean R. Runzel                                           */
--/*      Monmouth College, West Long Branch, NJ                            */
--/*               February 16, 1995                                        */
--/*                                                                        */
--/* Accompanies Public Domain YACC format Ada83 grammar                    */
--/*                                                                        */
--/* This work is based upon previous work performed by:                    */
--/*                                                                        */
--/* Herman Fischer  Litton Data Systems  1984                              */
--/* C. Schaefer                          1991                              */
--/* C. Howell                            1991                              */
--/* Richard Conn    Mitre, Inc.          1993                              */
--/*                 Intermetrics, Inc.   1994                              */
--/*                                                                        */
--/*------------------------------------------------------------------------*/
--! Fix by C. Schaefer for problem with attributes:
--! Thu Oct 24 13:23:00 EDT 1991
--!
--! Essentially, the lexer tries to make a character literal when it should
--! stop at a tic (apostrophe).  Because attribute names can be recursive
--! (e.g. t'base'last) and because attribute designators are mere (and
--! possibly single-character) identifiers, a parser must be able to deal
--! with sequences like
--! 
--!            a'b'c'd'e'f'...
--! 
--! In other words, no number of lookahead in the lexer is going to help.
--! The solution I am familiar with is to use syntatic context to help the
--! lexing.  In particular, the fix uses the rule, "If the last
--! (non-whitespace) token was an identifier, then a following tic must be a
--! tic token and not the first character of a character literal."  This
--! involves an apparently stupid state variable to remember what the last
--! token was.
--! 
--! Fix by C. Howell to recognize ^L (formfeed, == \f) as delimiter
--! Thu Oct 31 17:18:11 EST 1991
--! Changed [ \t]         {Bogus;}          --/* ignore spaces and tabs */
--! to [ \f\t]         {Bogus;}          --/* ignore spaces and tabs */


%%
-- Reserved Words
[aA][bB][oO][rR][tT]                { was_id := false; return(ABORT_TOKEN);}
[aA][bB][sS]                        { was_id := false; return(ABS_TOKEN);}
[aA][cC][cC][eE][pP][tT]            { was_id := false; return(ACCEPT_TOKEN);}
[aA][cC][cC][eE][sS][sS]            { was_id := false; return(ACCESS_TOKEN);}
[aA][lL][lL]                        { was_id := false; return(ALL_TOKEN);} 
[aA][nN][dD]                        { was_id := false; return(AND_TOKEN);} 
[aA][rR][rR][aA][yY]                { was_id := false; return(ARRAY_TOKEN);}
[aA][tT]                            { was_id := false; return(AT_TOKEN);}
[bB][eE][gG][iI][nN]                { was_id := false; return(BEGIN_TOKEN);}
[bB][oO][dD][yY]                    { was_id := false; return(BODY_TOKEN);}
[cC][aA][sS][eE]                    { was_id := false; return(CASE_TOKEN);}
[cC][oO][nN][sS][tT][aA][nN][tT]    { was_id := false;                      
                                      return(CONSTANT_TOKEN);}
[dD][eE][cC][lL][aA][rR][eE]        { was_id := false;                     
                                      return(DECLARE_TOKEN);}
[dD][eE][lL][aA][yY]                { was_id := false; return(DELAY_TOKEN);}
[dD][eE][lL][tT][aA]                { was_id := false; return(DELTA_TOKEN);}
[dD][iI][gG][iI][tT][sS]            { was_id := false; return(DIGITS_TOKEN);}
[dD][oO]                            { was_id := false; return(DO_TOKEN);}
[eE][lL][sS][eE]                    { was_id := false; return(ELSE_TOKEN);}
[eE][lL][sS][iI][fF]                { was_id := false; return(ELSIF_TOKEN);}
[eE][nN][dD]                        { was_id := false; return(END_TOKEN);}
[eE][nN][tT][rR][yY]                { was_id := false; return(ENTRY_TOKEN);}
[eE][xX][cC][eE][pP][tT][iI][oO][nN] { was_id := false;                     
                                       return(EXCEPTION_TOKEN);}
[eE][xX][iI][tT]                    { was_id := false; return(EXIT_TOKEN);}
[fF][oO][rR]                        { was_id := false; return(FOR_TOKEN);}
[fF][uU][nN][cC][tT][iI][oO][nN]    { was_id := false;                      
                                      return(FUNCTION_TOKEN);}
[gG][eE][nN][eE][rR][iI][cC]        { was_id := false; return(GENERIC_TOKEN);}
[gG][oO][tT][oO]                    { was_id := false; return(GOTO_TOKEN);}
[iI][fF]                            { was_id := false; return(IF_TOKEN);}
[iI][nN]                            { was_id := false; return(IN_TOKEN);}
[iI][sS]                            { was_id := false; return(IS_TOKEN);}
[lL][iI][mM][iI][tT][eE][dD]        { was_id := false; return(LIMITED_TOKEN);}
[lL][oO][oO][pP]                    { was_id := false; return(LOOP_TOKEN);}
[mM][oO][dD]                        { was_id := false; return(MOD_TOKEN);}
[nN][eE][wW]                        { was_id := false; return(NEW_TOKEN);}
[nN][oO][tT]                        { was_id := false; return(NOT_TOKEN);}
[nN][uU][lL][lL]                    { was_id := false; return(NULL_TOKEN);}
[oO][fF]                            { was_id := false; return(OF_TOKEN);}
[oO][rR]                            { was_id := false; return(OR_TOKEN);}
[oO][tT][hH][eE][rR][sS]            { was_id := false; return(OTHERS_TOKEN);}
[oO][uU][tT]                        { was_id := false; return(OUT_TOKEN);}
[pP][aA][cC][kK][aA][gG][eE]        { was_id := false; return(PACKAGE_TOKEN);}
[pP][rR][aA][gG][mM][aA]            { was_id := false; return(PRAGMA_TOKEN);}
[pP][rR][iI][vV][aA][tT][eE]        { was_id := false; return(PRIVATE_TOKEN);}
[pP][rR][oO][cC][eE][dD][uU][rR][eE] { was_id := false;                     
                                       return(PROCEDURE_TOKEN);}
[rR][aA][iI][sS][eE]                { was_id := false; return(RAISE_TOKEN);}
[rR][aA][nN][gG][eE]                { was_id := false; return(RANGE_TOKEN);}
[rR][eE][cC][oO][rR][dD]            { was_id := false; return(RECORD_TOKEN);}
[rR][eE][mM]                        { was_id := false; return(REM_TOKEN);}
[rR][eE][nN][aA][mM][eE][sS]        { was_id := false; return(RENAMES_TOKEN);}
[rR][eE][tT][uU][rR][nN]            { was_id := false; return(RETURN_TOKEN);}
[rR][eE][vV][eE][rR][sS][eE]        { was_id := false; return(REVERSE_TOKEN);}
[sS][eE][lL][eE][cC][tT]            { was_id := false; return(SELECT_TOKEN);}
[sS][eE][pP][aA][rR][aA][tT][eE]    { was_id := false;                      
                                      return(SEPARATE_TOKEN);}
[sS][uU][bB][tT][yY][pP][eE]        { was_id := false; return(SUBTYPE_TOKEN);}
[tT][aA][sS][kK]                    { was_id := false; return(TASK_TOKEN);}
[tT][eE][rR][mM][iI][nN][aA][tT][eE] { was_id := false;                     
                                       return(TERMINATE_TOKEN);}
[tT][hH][eE][nN]                    { was_id := false; return(THEN_TOKEN);}
[tT][yY][pP][eE]                    { was_id := false; return(TYPE_TOKEN);}
[uU][sS][eE]                        { was_id := false; return(USE_TOKEN);}
[wW][hH][eE][nN]                    { was_id := false; return(WHEN_TOKEN);}
[wW][hH][iI][lL][eE]                { was_id := false; return(WHILE_TOKEN);}
[wW][iI][tT][hH]                    { was_id := false; return(WITH_TOKEN);}
[xX][oO][rR]                        { was_id := false; return(XOR_TOKEN);}

-- Delimiters
"=>"    { was_id := false; return(ARROW);}
".."    { was_id := false; return(DOUBLE_DOT);}
"**"    { was_id := false; return(DOUBLE_STAR);}
":="    { was_id := false; return(ASSIGNMENT);}
"/="    { was_id := false; return(INEQUALITY);}
">="    { was_id := false; return(GREATER_THAN_OR_EQUAL);}
"<="    { was_id := false; return(LESS_THAN_OR_EQUAL);}
"<<"    { was_id := false; return(LEFT_LABEL_BRACKET);}
">>"    { was_id := false; return(RIGHT_LABEL_BRACKET);}
"<>"    { was_id := false; return(BOX);}
"&"     { was_id := false; return('&'); }
"("     { was_id := false; return('('); }
")"     { was_id := false; return(')'); }
"*"     { was_id := false; return('*'); }
"+"     { was_id := false; return('+'); }
","     { was_id := false; return(','); }
"-"     { was_id := false; return('-'); }
"."     { was_id := false; return('.'); }
"/"     { was_id := false; return('/'); }
":"     { was_id := false; return(':'); }
";"     { was_id := false; return(';'); }
"<"     { was_id := false; return('<'); }
"="     { was_id := false; return('='); }
">"     { was_id := false; return('>'); }
"|"     { was_id := false; return('|'); }
"!"     { was_id := false; return('|'); } -- Ada Reference Manual 2.10(1)
"'"     { was_id := false; return('''); }

-- Literals
{IDENTIFIER}          { was_id := true; return(IDENTIFIER); }
{DECIMAL_LITERAL}     { was_id := false; return(NUMERIC_LITERAL);}
{BASED_LITERAL}       { was_id := false; return(NUMERIC_LITERAL);}
{STRING_LITERAL}      { was_id := false; return(STRING_LITERAL);}
{CHAR_LITERAL}        {
                        --| kludge for character literal / attribute problem
                        if was_id then
                          yyless(1);
                          was_id := false;
                          return (''');
                        else
                          was_id := false;
                          return(CHARACTER_LITERAL);
                        end if;
                      }

-- Whitespace and Comments
[ \f\t\r]     {Bogus;}          --/* ignore spaces, tabs, and returns */
"--".*        {Bogus;}          --/* ignore comments */
[\n]          {Lines := Lines + 1;}

-- Errors 
.             {return(ERROR);}

%%

-- ***********************************************************
-- *                                                         *
-- * Ada83_Lexical_Analyzer                                  * SPEC
-- *                                                         *
-- ***********************************************************
package A83 is
--| PURPOSE
--| Analyze a stream of characters for tokens, producing a stream of
--| tokens as a result.  The input stream is obtained through Text_IO.
--| The output stream is obtained by the user of this package through
--| successive calls to Get_Token.
--|
--| AUTHOR AND REVISION HISTORY:
--| Version  Date        Author             Notes
--|   1.0    12 Nov 93   Richard Conn       Initial release
--|   1.1     6 Mar 95   Dean Runzel        Change package name and correct
--|                                         identifier errors
--|   1.2.    2 Apr 96   Richard Conn       Modify for SCATC Kit
--| NOTES:
--| The sequence of invocation of the methods in this package is:
--|     Reset - initialize analyzer and open input file
--|     Get_Token -- as desired until END_OF_INPUT is returned
--|     Get_Token_Value -- as desired (returns string of token)

  type Token is
      (End_Of_Input, Error, '&', ''',
       '(', ')', '*',
       '+', ',', '-',
       '.', '/', ':',
       ';', '<', '=',
       '>', '|', Arrow,
       Double_Dot, Double_Star, Assignment,
       Inequality, Greater_Than_Or_Equal, Less_Than_Or_Equal,
       Left_Label_Bracket, Right_Label_Bracket, Box,
       Abort_Token, Abs_Token, Accept_Token,
       Access_Token, All_Token, And_Token,
       Array_Token, At_Token, Begin_Token,
       Body_Token, Case_Token, Constant_Token,
       Declare_Token, Delay_Token, Delta_Token,
       Digits_Token, Do_Token, Else_Token,
       Elsif_Token, End_Token, Entry_Token,
       Exception_Token, Exit_Token, For_Token,
       Function_Token, Generic_Token, Goto_Token,
       If_Token, In_Token, Is_Token,
       Limited_Token, Loop_Token, Mod_Token,
       New_Token, Not_Token, Null_Token,
       Of_Token, Or_Token, Others_Token,
       Out_Token, Package_Token, Pragma_Token,
       Private_Token, Procedure_Token, Raise_Token,
       Range_Token, Record_Token, Rem_Token,
       Renames_Token, Return_Token, Reverse_Token,
       Select_Token, Separate_Token, Subtype_Token,
       Task_Token, Terminate_Token, Then_Token,
       Type_Token, Use_Token, When_Token,
       While_Token, With_Token, Xor_Token,
       Identifier, Numeric_Literal, Character_Literal,
       String_Literal );

  procedure Reset (File_Name : in STRING);
  --| PURPOSE
  --| Zero line number count for current file and update total line
  --| count for all files.  Reset key variables for current file.
  --| Open input and output files.

  procedure Close_Files;
  --| PURPOSE
  --| Close the input and output files.

  function Get_Token return TOKEN;
  --| PURPOSE
  --| Return current token

  function Get_Token_Value return STRING;
  --| PURPOSE
  --| Return value associated with current token (e.g., if current
  --| token is an identifier, this is the name of the identifier);
  --| "Current token" is the last token returned by a call to
  --| Get_Token

  function Get_Current_Line return NATURAL;
  --| PURPOSE
  --| Return line on which current token is located

  procedure Get_Statistics (Number_of_Lines : out NATURAL);
  --| PURPOSE
  --| Return statistics on the last file processed.  These
  --| counters are reset by a call to Reset.  This
  --| routine should be called after the parse is complete.

end A83;

-- ***********************************************************
-- *                                                         *
-- * Ada83_Lexical_Analyzer                                  * BODY
-- *                                                         *
-- ***********************************************************
with Text_IO; use Text_IO;
package body A83 is

------------------------------------------------------------------
-- Local types and objects
------------------------------------------------------------------

  Lines            : NATURAL := 1;
  Was_ID           : BOOLEAN := FALSE;

------------------------------------------------------------------
-- Local routines
------------------------------------------------------------------

  -- ***********************************************************
  -- *                                                         *
  -- * Lexical_Analyzer.Bogus                                  *
  -- *                                                         *
  -- ***********************************************************
  procedure Bogus is
    -- dummy routine
  begin
    null;
  end Bogus;

------------------------------------------------------------------
-- Routines exported by the specification
------------------------------------------------------------------
  -- ***********************************************************
  -- *                                                         *
  -- * Ada83_Lexical_Analyzer.Reset                            *
  -- *                                                         *
  -- ***********************************************************
  procedure Reset (File_Name : in STRING) is
  --| PURPOSE
  --| Zero line number count and reset key variables for current
  --| file.  Open input and output files.
  begin -- Reset
    Lines           := 1;
    Was_ID          := FALSE;
    A83_IO.Open_Input (File_Name);
    A83_IO.Create_Output;
  end Reset;

  -- ***********************************************************
  -- *                                                         *
  -- * Ada83_Lexical_Analyzer.Close_Files                      *
  -- *                                                         *
  -- ***********************************************************
  procedure Close_Files is
  --| PURPOSE
  --| Close the input and output files.
  begin -- Close_Files
    A83_IO.Close_Input;
    A83_IO.Close_Output;
  end Close_Files;
  
  -- ***********************************************************
  -- *                                                         *
  -- * Ada83_Lexical_Analyzer.Get_Token                        *
  -- *                                                         *
  -- ***********************************************************
  function YYLex return TOKEN; -- Forward reference for YYLex
  function Get_Token return TOKEN is
  --| PURPOSE
  --| Return current token
  begin -- Get_Token
    return YYLex;
  end Get_Token;

  --| NOTE:
  --| This function is acronymic and does not reflect a good design
  --| choice, but it is retained for compatibility with the YYParse
  --| function generated by AYACC, where YYParse calls YYLex.
  --|
  --| Function YYLex is inserted here by AFLEX.
  --| Return current token (inserted below).

##

  -- ***********************************************************
  -- *                                                         *
  -- * Ada83_Lexical_Analyzer.Get_Token_Value                  *
  -- *                                                         *
  -- ***********************************************************
  function Get_Token_Value return STRING is
  --| PURPOSE
  --| Return value associated with current token (e.g., if current
  --| token is an identifier, this is the name of the identifier);
  --| "Current token" is the last token returned by a call to
  --| Get_Token
  begin -- Get_Token_Value
    return A83_DFA.YYText;
  end Get_Token_Value;

  -- ***********************************************************
  -- *                                                         *
  -- * Ada83_Lexical_Analyzer.Get_Current_Line                 *
  -- *                                                         *
  -- ***********************************************************
  function Get_Current_Line return NATURAL is
  --| PURPOSE
  --| Return line on which current token is located
  begin -- Get_Current_Line
    return Lines;
  end Get_Current_Line;

  -- ***********************************************************
  -- *                                                         *
  -- * Ada83_Lexical_Analyzer.Get_Statistics                   *
  -- *                                                         *
  -- ***********************************************************
  procedure Get_Statistics (Number_of_Lines : out NATURAL) is
  --| PURPOSE
  --| Return statistics on the last file processed.  These
  --| counters are reset by a call to Reset_Linenum.  This
  --| routine should be called after the parse is complete.
  begin -- Get_Statistics
    Number_of_Lines := Lines - 1;
  end Get_Statistics;

end A83;

