-- **********************************************
-- *                                            *
-- * Command_Line_Processor                     * BODY
-- *                                            *
-- **********************************************
with Command_Line_Interface;
with Information_Collector;
with Text_IO;
package body Command_Line_Processor is

  File_Name_Count : NATURAL := 0;

  type String_Pointer is access STRING;

  Include_File_Prefix  : constant CHARACTER := '@';
  Inline_Buffer_Length : constant := 400;

  function Less_Than (L, R : in String_Pointer) return BOOLEAN is
  begin -- Less_Than
    return L.all < R.all;
  end Less_Than;

  package File_Name_Collector is new Information_Collector
    (Information_Packet => String_Pointer,
     "<"                => Less_Than);

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Reset                *
  --*                                             *
  --***********************************************
  procedure Reset is
  begin -- Reset
    File_Name_Collector.Reset_Iterator;
  end Reset;

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Number_of_File_Names *
  --*                                             *
  --***********************************************
  function Number_of_File_Names return NATURAL is
  begin -- Number_of_File_Names
    return File_Name_Count;
  end Number_of_File_Names;

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Is_End_of_File_Names *
  --*                                             *
  --***********************************************
  function Is_End_of_File_Names return BOOLEAN is
  begin -- Is_End_of_File_Names
    return File_Name_Collector.Is_End_of_Packets;
  end Is_End_of_File_Names;

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Next_File_Name       *
  --*                                             *
  --***********************************************
  function Next_File_Name return STRING is
    SP : String_Pointer;
  begin -- Next_File_Name
    if File_Name_Collector.Is_End_of_Packets then
      raise END_OF_FILE_LIST;
    end if;
    File_Name_Collector.Fetch_Next_Information_Packet (SP);
    return SP.all;
  end Next_File_Name;

  --***********************************************
  --*                                             *
  --* Command_Line_Processor.Process_File_Name    *
  --*                                             *
  --***********************************************
  procedure Process_File_Name (Name : in STRING) is

    --*****************************************************************
    --*                                                               *
    --* Command_Line_Processor.Process_File_Name.Trim                 *
    --*                                                               *
    --*****************************************************************
    function Trim (S : in STRING) return STRING is
      Start : NATURAL := S'FIRST;
      Stop  : NATURAL := S'FIRST-1; -- null string
      type STATE is (LOOKING_FOR_FIRST, LOOKING_FOR_LAST);
      Current_State : STATE := LOOKING_FOR_FIRST;
    begin -- Trim
      for I in S'range loop
        case Current_State is
          when LOOKING_FOR_FIRST =>
            if S(I) > ' ' then
              Start := I;
              Stop  := I;
              Current_State := LOOKING_FOR_LAST;
            end if;
          when LOOKING_FOR_LAST =>
            if S(I) > ' ' then
              Stop := I;
            end if;
        end case;
      end loop;
      return S(Start .. Stop);
    end Trim;

    --*****************************************************************
    --*                                                               *
    --* Command_Line_Processor.Process_File_Name.Process_Regular_File *
    --*                                                               *
    --*****************************************************************
    procedure Process_Regular_File (Name : in STRING) is
      SP : String_Pointer;
    begin
      SP := new STRING'(Name);
      File_Name_Collector.Add_Information_Packet (SP);
      File_Name_Count := File_Name_Count + 1;
    end Process_Regular_File;

    --*****************************************************************
    --*                                                               *
    --* Command_Line_Processor.Process_File_Name.Process_Include_File * SPEC
    --*                                                               *
    --*****************************************************************
    procedure Process_Include_File (Name : in STRING);

    --**********************************************************************
    --*                                                                    *
    --* Command_Line_Processor.Process_File_Name.Process_Include_File_Line *
    --*                                                                    *
    --**********************************************************************
    procedure Process_Include_File_Line (Line : in STRING) is
    begin -- Process_Include_File_Line
      if Line'LENGTH > 0 then -- skip empty lines
        if Line'LENGTH >= 2 and then
           Line(Line'FIRST .. Line'FIRST+1) = "--" then -- skip comments
          null;
        else
          if Line(Line'FIRST) = Include_File_Prefix then
            Process_Include_File (Line(Line'FIRST+1 .. Line'LAST));
          else
            Process_Regular_File (Line);
          end if;
        end if;
      end if;
    end Process_Include_File_Line;

    --*****************************************************************
    --*                                                               *
    --* Command_Line_Processor.Process_File_Name.Process_Include_File *
    --*                                                               *
    --*****************************************************************
    procedure Process_Include_File (Name : in STRING) is
      File_ID : Text_IO.FILE_TYPE;
      Inline  : STRING(1..Inline_Buffer_Length);
      Inlast  : NATURAL;
    begin -- Process_Include_File
      Text_IO.Open (File_ID, Text_IO.IN_FILE, Name);
      while not Text_IO.End_of_File (File_ID) loop
        Text_IO.Get_Line (File_ID, Inline, Inlast);
        Process_Include_File_Line (Trim(Inline(1..Inlast)));
      end loop;
      Text_IO.Close (File_ID);
    end Process_Include_File;

    --*****************************************************************
    --*                                                               *
    --* Command_Line_Processor.Process_File_Name (body)               *
    --*                                                               *
    --*****************************************************************
  begin -- Process_File_Name
    if Name'LENGTH > 0 then
      if Name(Name'FIRST) = Include_File_Prefix then
        Process_Include_File (Name(Name'FIRST+1 .. Name'LAST));
      else
        Process_Regular_File (Name);
      end if;
    end if;
  end Process_File_Name;

begin -- Initialize section

  for I in 1 .. Command_Line_Interface.Token_Count - 1 loop
    Process_File_Name (Command_Line_Interface.Token_Value(I));
  end loop;
  Reset;

end Command_Line_Processor;
