-- *********************************************************
-- *                                                       *
-- *  Console                                              *  BODY
-- *                                                       *
-- *********************************************************
with Ada.Text_IO;
with Ada.Unchecked_Conversion;
package body Console is
--| Notes (none)

  type STATE is (DISABLED, ENABLED);

  Output_State : array (1..Max_Number_of_States) of STATE
    := (others => ENABLED);

  Current_State : NATURAL := 1;

  Terminal : TERMINAL_KIND := TTY;

  package INTIO is new Ada.Text_IO.INTEGER_IO(INTEGER);
  package FLTIO is new Ada.Text_IO.FLOAT_IO(FLOAT);

  function Rend_to_Int is new Ada.Unchecked_Conversion (RENDITION,
                                                    INTEGER);

  -- .................................................................
  -- .                                                               .
  -- .  Console.Set_Terminal                                         .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Set_Terminal (New_Setting : in TERMINAL_KIND := TTY) is
  --| Notes (none)
  begin
    Terminal := New_Setting;
  end Set_Terminal;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Enable_Output                                        .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Enable_Output is
  --| Notes (none)
  begin
    Output_State(Current_State) := ENABLED;
  end Enable_Output;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Disable_Output                                       .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Disable_Output is
  --| Notes (none)
  begin
    Output_State(Current_State) := DISABLED;
  end Disable_Output;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Push                                                 .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Push is
  --| Notes (none)
  begin
    if Current_State = Max_Number_of_States then
      raise STATE_OVERFLOW;
    else
      Current_State := Current_State + 1;
    end if;
  end Push;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Pop                                                  .  BODY   
  -- .                                                               .
  -- .................................................................
  procedure Pop is
  --| Notes (none)
  begin
    if Current_State = Output_State'FIRST then
      raise STATE_UNDERFLOW;
    else
      Current_State := Current_State - 1;
    end if;
  end Pop;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Position_Cursor                                      .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Position_Cursor (Row    : in ROW_NUMBER;
                             Column : in COLUMN_NUMBER) is
  --| Notes (none)
  begin
    if (Terminal /= TTY) and (Output_State(Current_State) = ENABLED) then
      Ada.Text_IO.Put (ASCII.ESC & "[");
      INTIO.Put (INTEGER(Row), 0);
      Ada.Text_IO.Put (';');
      INTIO.Put (INTEGER(Column), 0);
      Ada.Text_IO.Put ('H');
    end if;
  end Position_Cursor;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Erase_Display                                        .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Erase_Display is
  --| Notes (none)
  begin
    if (Terminal /= TTY) and (Output_State(Current_State) = ENABLED) then
      Ada.Text_IO.Put (ASCII.ESC & "[2J");
    end if;
  end Erase_Display;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Erase_Line                                           .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Erase_Line is
  --| Notes (none)
  begin
    if (Terminal /= TTY) and (Output_State(Current_State) = ENABLED) then
      Ada.Text_IO.Put (ASCII.ESC & "[K");
    end if;
  end Erase_Line;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Set_Rendition                                        .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Set_Rendition (New_Setting : in RENDITION) is
  --| Notes
  --|   If the value of Terminal is VT100, no action is taken for any
  --| of the color renditions.
  begin
    if Output_State(Current_State) = ENABLED then
      case New_Setting is
        when ALL_ATTRIBUTES_OFF =>
          if Terminal = ANSI then
            Ada.Text_IO.Put (ASCII.ESC & "[0m");
          elsif Terminal = VT100 then
            Ada.Text_IO.Put (ASCII.ESC & "[m");
          end if;
        when HIGH_INTENSITY | BLINKING | REVERSE_VIDEO =>
          if Terminal /= TTY then
            Ada.Text_IO.Put (ASCII.ESC & "[");
            INTIO.Put (Rend_to_Int(New_Setting), 0);
            Ada.Text_IO.Put ('m');
          end if;
        when others =>
          if Terminal = ANSI then
            Ada.Text_IO.Put (ASCII.ESC & "[");
            INTIO.Put (Rend_to_Int(New_Setting), 0);
            Ada.Text_IO.Put ('m');
          end if;
      end case;
    end if;
  end Set_Rendition;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Put                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Put (Item : in CHARACTER) is
  --| Notes (none)
  begin
    if Output_State(Current_State) = ENABLED then
      Ada.Text_IO.Put (Item);
    end if;
  end Put;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Put                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Put (Item : in STRING) is
  --| Notes (none)
  begin
    if Output_State(Current_State) = ENABLED then
      Ada.Text_IO.Put (Item);
    end if;
  end Put;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Put                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Put
    ( Item           : in STRING;
      Field_Width    : in NATURAL;
      On_Overflow    : in OVERFLOW_ACTION := TRUNCATE_TAIL;
      On_Underflow   : in JUSTIFICATION   := LEFT_JUSTIFIED;
      Fill_Char      : in CHARACTER       := ' ';
      Overflow_Char  : in CHARACTER       := '*' ) is
  --| Notes (none)
    First_Char  : NATURAL := Item'first;
    Last_Char   : NATURAL := Item'last;
    Fill_Width  : NATURAL;
  begin
    if Output_State(Current_State) = ENABLED then
      if Item'length > Field_Width then
        case On_Overflow is
          when TRUNCATE_TAIL           =>
            Last_Char  := First_Char + Field_Width - 1;
            Ada.Text_IO.Put (Item(First_Char .. Last_Char));
          when TRUNCATE_HEAD           =>
            First_Char := Last_Char - Field_Width + 1;
            Ada.Text_IO.Put (Item(First_Char .. Last_Char));
          when FILL_WITH_OVERFLOW_CHAR =>
            for I in 1 .. Field_Width loop
              Ada.Text_IO.Put (Overflow_Char);
            end loop;
        end case;
      elsif Item'length < Field_Width then
        Fill_Width := Field_Width - Item'length;
        case On_Underflow is
          when LEFT_JUSTIFIED  =>
            Ada.Text_IO.Put (Item);
            for I in 1 .. Fill_Width loop
              Ada.Text_IO.Put (Fill_Char);
            end loop;
          when RIGHT_JUSTIFIED =>
            for I in 1 .. Fill_Width loop
              Ada.Text_IO.Put (Fill_Char);
            end loop;
            Ada.Text_IO.Put (Item);
        end case;
      else
        Ada.Text_IO.Put (Item);
      end if;
    end if;
  end Put;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Put                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Put (Item          : in INTEGER;
                 Width         : in NATURAL;
                 On_Overflow   : in NUMERIC_OVERFLOW_ACTION
                                   := FILL_WITH_OVERFLOW_CHAR;
                 Overflow_Char : in CHARACTER := '*') is
  --| Notes (none)
    Overflow : BOOLEAN := FALSE;
  begin
    if Output_State(Current_State) = ENABLED then
      begin
        if Width = 0 then
          Overflow := TRUE;
        else
          if Item < 0 then
            if Item <= -10**(Width-1) then
              Overflow := TRUE;
            end if;
          else
            if Item >= 10**Width then
              Overflow := TRUE;
            end if;
          end if;
        end if;
      exception
        when others =>
          Overflow := FALSE;
      end;
      if not Overflow then
        INTIO.Put (Item, Width);
      else -- Overflow
        case On_Overflow is
          when FILL_WITH_OVERFLOW_CHAR =>
            for I in 1 .. Width loop
              Ada.Text_IO.Put (Overflow_Char);
            end loop;
          when OUTPUT_FULL_NUMBER             =>
            INTIO.Put (Item, Width);
        end case;
      end if;
    end if;
  end Put;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Put                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Put (Item          : in FLOAT;
                 Fore          : in NATURAL;
                 Aft           : in NATURAL;
                 On_Overflow   : in NUMERIC_OVERFLOW_ACTION
                                   := FILL_WITH_OVERFLOW_CHAR;
                 Overflow_Char : in CHARACTER := '*') is
  --| Notes (none)
    Overflow : BOOLEAN := FALSE;
  begin
    if Output_State(Current_State) = ENABLED then
      begin
        if Fore = 0 then
          Overflow := TRUE;
        else
          if Item < 0.0 then
            if Item <= -10.0**(Fore-1) then
              Overflow := TRUE;
            end if;
          else
            if Item >= 10.0**Fore then
              Overflow := TRUE;
            end if;
          end if;
        end if;
      exception
        when others =>
          Overflow := FALSE;
      end;
      if not Overflow then
        FLTIO.Put (Item, Fore, Aft, 0);
      else -- Overflow
        case On_Overflow is
          when FILL_WITH_OVERFLOW_CHAR =>
            for I in 1 .. Fore loop
              Ada.Text_IO.Put (Overflow_Char);
            end loop;
            Ada.Text_IO.Put (Overflow_Char); -- decimal
            for I in 1 .. Aft loop
              Ada.Text_IO.Put (Overflow_Char);
            end loop;
          when OUTPUT_FULL_NUMBER             =>
            FLTIO.Put (Item, Fore, Aft, 0);
        end case;
      end if;
    end if;
  end Put;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Put                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Put (Item : in FLOAT;
                 Fore : in NATURAL := 2;
                 Aft  : in NATURAL := 2;
                 Exp  : in NATURAL := 3) is
  --| Notes (none)
  begin
    if Output_State(Current_State) = ENABLED then
      FLTIO.Put (Item, Fore, Aft, Exp);
    end if;
  end Put;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Put_Line                                             .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Put_Line (Item : in STRING) is
  --| Notes (none)
  begin
    if Output_State(Current_State) = ENABLED then
      Ada.Text_IO.Put_Line (Item);
    end if;
  end Put_Line;

  -- .................................................................
  -- .                                                               .
  -- .  Console.New_Line                                             .  BODY
  -- .                                                               .
  -- .................................................................
  procedure New_Line is
  --| Notes (none)
  begin
    if Output_State(Current_State) = ENABLED then
      Ada.Text_IO.New_Line;
    end if;
  end New_Line;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Get                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Get
    ( Item             : out CHARACTER ) is
  --| Notes (none)
  begin -- Get
    Ada.Text_IO.Get (Item);
  exception
    when others =>
      raise INPUT_ERROR;
  end Get;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Get                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Get
    ( Item             : out INTEGER ) is
  --| Notes (none)
  begin -- Get
    INTIO.Get (Item);
  exception
    when others =>
      raise INPUT_ERROR;
  end Get;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Get                                                  .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Get
    ( Item             : out FLOAT ) is
  --| Notes (none)
  begin -- Get
    FLTIO.Get (Item);
  exception
    when others =>
      raise INPUT_ERROR;
  end Get;

  -- .................................................................
  -- .                                                               .
  -- .  Console.Get_Line                                             .  BODY
  -- .                                                               .
  -- .................................................................
  procedure Get_Line
    ( Item           : out STRING;
      Last           : out NATURAL ) is
  --| Notes (none)
  begin -- Get_Line
    Ada.Text_IO.Get_Line(Item, Last);
  end Get_Line;

end Console;
