-- *******************************************************************
-- *                                                                 *
-- * Report_Generator                                                * BODY
-- *                                                                 *
-- *******************************************************************
with Ada.Text_IO;
package body Report_Generator is

  RFD : Ada.Text_IO.FILE_TYPE; -- Report File Descriptor
  CFD : Ada.Text_IO.FILE_TYPE; -- Contents File Descriptor

  type FILE_STATE is (LOOKING_FOR_FIRST_LINE_ON_PAGE, ON_PAGE);
  Report_State   : FILE_STATE  := LOOKING_FOR_FIRST_LINE_ON_PAGE;
  Contents_State : FILE_STATE  := LOOKING_FOR_FIRST_LINE_ON_PAGE;
  Lines_Left_on_Report_Page    : NATURAL := Bottom_Margin - Top_Margin - 2;
  Lines_Left_on_Contents_Page  : NATURAL := Bottom_Margin - Top_Margin - 2;
  Current_Left_Margin          : NATURAL := Left_Margin;
  Current_Report_Page_Number   : NATURAL := 1;
  Current_Contents_Page_Number : NATURAL := 1;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Center                                         *
  -- *                                                                 *
  -- *******************************************************************
  function Center (Item : in STRING) return STRING is
  --| PURPOSE
  --| Center the Item such that the left margin is applied.
    Centered_Line : STRING (1..Maximum_Line_Length) :=
                      (others => ' ');
    First, Last      : NATURAL;
  begin -- Center
    if Item'LENGTH > Right_Margin - Current_Left_Margin then
      First := Current_Left_Margin + 1;
    else
      First := Current_Left_Margin +
               (Right_Margin - Current_Left_Margin - Item'LENGTH)/2;
    end if;
    Last  := First + Item'LENGTH - 1;
    Centered_Line (First .. Last) := Item;
    return Centered_Line (1..Last);
  end Center;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Left                                           *
  -- *                                                                 *
  -- *******************************************************************
  function Left (Item               : in STRING;
                 Append_Page_Number : in BOOLEAN := FALSE) return STRING is
  --| PURPOSE
  --| Left-Justify the Item such that the left margin is applied.
  --| If Append_Page_Number is TRUE, place a page number in the line
  --| as a right-justified value.

    LJ_Line      : STRING (1..Maximum_Line_Length) :=
                     (others => ' ');
    Last         : NATURAL;
    Indent_Value : NATURAL := Current_Report_Page_Number;
    Indent_By    : NATURAL := 1; -- for leading space
  begin -- Left
    Last := Current_Left_Margin + Item'LENGTH; -- index of last char
    LJ_Line (Current_Left_Margin+1 .. Last) := Item;
    if Append_Page_Number then
      while Indent_Value > 10 loop
        Indent_By    := Indent_By + 1;
        Indent_Value := Indent_Value / 10;
      end loop;
      LJ_Line (Right_Margin - Indent_By .. Right_Margin) :=
        NATURAL'IMAGE(Current_Report_Page_Number);
      Last := Right_Margin;
    end if;
    return LJ_Line (1 .. Last);
  end Left;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Start_New_Report_Page                          *
  -- *                                                                 *
  -- *******************************************************************
  procedure Start_New_Report_Page is
  begin -- Start_New_Report_Page
    for I in 1 .. Top_Margin loop
      Ada.Text_IO.New_Line (RFD);
    end loop;
    Ada.Text_IO.Put_Line (RFD, Center(Page_Header_Line));
    Ada.Text_IO.New_Line (RFD);
    Lines_Left_on_Report_Page := Bottom_Margin - Top_Margin - 2;
  end Start_New_Report_Page;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Start_New_Contents_Page                        *
  -- *                                                                 *
  -- *******************************************************************
  procedure Start_New_Contents_Page is
  begin -- Start_New_Contents_Page
    for I in 1 .. Top_Margin loop
      Ada.Text_IO.New_Line (CFD);
    end loop;
    Ada.Text_IO.Put_Line (CFD, Center(Page_Header_Line));
    Ada.Text_IO.New_Line (CFD);
    Lines_Left_on_Contents_Page := Bottom_Margin - Top_Margin - 2;
  end Start_New_Contents_Page;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Create_Report_and_Contents_Files               *
  -- *                                                                 *
  -- *******************************************************************
  procedure Create_Report_and_Contents_Files is
  begin -- Create_Report_and_Contents_Files
    Ada.Text_IO.Create (RFD, Ada.Text_IO.OUT_FILE, Report_File_Name);
    Ada.Text_IO.Create (CFD, Ada.Text_IO.OUT_FILE, Contents_File_Name);
  exception
    when others =>
      raise FILE_CREATION_ERROR;
  end Create_Report_and_Contents_Files;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Put_Line_to_Report_File                        *
  -- *                                                                 *
  -- *******************************************************************
  procedure Put_Line_to_Report_File
      (Line            : in STRING  := "";
       Center_the_Line : in BOOLEAN := FALSE;
       Indent_by       : in NATURAL := 0) is
    Last : NATURAL;
  begin -- Put_Line_to_Report_File
    if Indent_by <= Left_Margin then
      Current_Left_Margin := Left_Margin - Indent_by;
    else
      Current_Left_Margin := 0;
    end if;
    if Report_State = LOOKING_FOR_FIRST_LINE_ON_PAGE then
      Start_New_Report_Page;
      Report_State := ON_PAGE;
    end if;
    if Lines_Left_on_Report_Page = 0 then
      Eject_Page_in_Report_File;
      Start_New_Report_Page;
      Put_Line_to_Report_File (Line, Center_the_Line, Indent_by);
    else
      Lines_Left_on_Report_Page := Lines_Left_on_Report_Page - 1;
      if Line'LENGTH <= Right_Margin - Current_Left_Margin then -- line fits
        if Center_the_Line then
          Ada.Text_IO.Put_Line (RFD, Center(Line)); -- output centered line
        else
          Ada.Text_IO.Put_Line (RFD, Left(Line));   -- output left-justified line
        end if;
      else -- line does not fit
        Last := Line'FIRST + (Right_Margin - Current_Left_Margin) - 1;
        Ada.Text_IO.Put_Line (RFD, Left(Line(Line'FIRST .. Last)) & " -");
        Put_Line_to_Report_File (Line(Last+1 .. Line'LAST), FALSE, 0);
      end if;
    end if;
  exception
    when others =>
      raise FILE_WRITE_ERROR;
  end Put_Line_to_Report_File;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Put_Line_to_Contents_File                      *
  -- *                                                                 *
  -- *******************************************************************
  procedure Put_Line_to_Contents_File
      (Line               : in STRING  := "";
       Center_the_Line    : in BOOLEAN := FALSE;
       Append_Page_Number : in BOOLEAN := FALSE) is
    Last : NATURAL;
  begin -- Put_Line_to_Contents_File
    Current_Left_Margin := Left_Margin;
    if Contents_State = LOOKING_FOR_FIRST_LINE_ON_PAGE then
      Start_New_Contents_Page;
      Contents_State := ON_PAGE;
    end if;
    if Lines_Left_on_Contents_Page = 0 then
      Eject_Page_in_Contents_File;
      Start_New_Contents_Page;
      Put_Line_to_Contents_File (Line, Center_the_Line, Append_Page_Number);
    else
      Lines_Left_on_Contents_Page := Lines_Left_on_Contents_Page - 1;
      if Line'LENGTH <= Right_Margin - Current_Left_Margin then -- line fits
        if Center_the_Line then
          Ada.Text_IO.Put_Line (CFD, Center(Line)); -- output centered line
        else
          Ada.Text_IO.Put_Line (CFD, Left(Line, Append_Page_Number));
            -- output left-justified line
        end if;
      else -- line does not fit
        Last := Line'FIRST + (Right_Margin - Current_Left_Margin) - 1;
        Ada.Text_IO.Put_Line (CFD, Left(Line(Line'FIRST .. Last)) & " -");
        Put_Line_to_Contents_File (Line(Last+1 .. Line'LAST), FALSE,
                                   Append_Page_Number);
      end if;
    end if;
  exception
    when others =>
      raise FILE_WRITE_ERROR;
  end Put_Line_to_Contents_File;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Eject_Page_in_Report_File                      *
  -- *                                                                 *
  -- *******************************************************************
  procedure Eject_Page_in_Report_File is
  begin -- Eject_Page_in_Report_File
    if Lines_Left_on_Report_Page > 0 then
      for I in 1 .. Lines_Left_on_Report_Page loop
        Ada.Text_IO.New_Line (RFD);
      end loop;
    end if;
    Ada.Text_IO.New_Line (RFD);
    Current_Left_Margin := Left_Margin;
    Ada.Text_IO.Put_Line (RFD,
                      Center(NATURAL'IMAGE(Current_Report_Page_Number)));
    Current_Report_Page_Number := Current_Report_Page_Number + 1;
    Ada.Text_IO.Put (RFD, ASCII.FF);
  exception
    when others =>
      raise FILE_WRITE_ERROR;
  end Eject_Page_in_Report_File;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Eject_Page_in_Contents_File                    *
  -- *                                                                 *
  -- *******************************************************************
  procedure Eject_Page_in_Contents_File is
  begin -- Eject_Page_in_Contents_File
    if Lines_Left_on_Contents_Page > 0 then
      for I in 1 .. Lines_Left_on_Contents_Page loop
        Ada.Text_IO.New_Line (CFD);
      end loop;
    end if;
    Ada.Text_IO.New_Line (CFD);
    Current_Left_Margin := Left_Margin;
    Ada.Text_IO.Put_Line (CFD,
                      Center(NATURAL'IMAGE(Current_Contents_Page_Number)));
    Current_Contents_Page_Number := Current_Contents_Page_Number + 1;
    Ada.Text_IO.Put (CFD, ASCII.FF);
  exception
    when others =>
      raise FILE_WRITE_ERROR;
  end Eject_Page_in_Contents_File;

  -- *******************************************************************
  -- *                                                                 *
  -- * Report_Generator.Close_Report_and_Contents_Files                *
  -- *                                                                 *
  -- *******************************************************************
  procedure Close_Report_and_Contents_Files is
  begin -- Close_Report_and_Contents_Files
    Eject_Page_in_Report_File;
    Eject_Page_in_Contents_File;
    Ada.Text_IO.Close (RFD);
    Ada.Text_IO.Close (CFD);
  exception
    when others =>
      raise FILE_WRITE_ERROR;
  end Close_Report_and_Contents_Files;

end Report_Generator;
