-- *******************************************************************
-- *                                                                 *
-- * Report_Generator                                                * SPEC
-- *                                                                 *
-- *******************************************************************
generic
  Report_File_Name    : STRING  := "report.txt";
  Contents_File_Name  : STRING  := "contents.txt";
  Page_Header_Line    : STRING  := "Source Code Analysis Report";
  Left_Margin         : NATURAL := 10; -- character position in line
  Right_Margin        : NATURAL := 70; -- character position in line
  Top_Margin          : NATURAL :=  6; -- number of blank lines at top of page
  Bottom_Margin       : NATURAL := 60; -- last printed line on page
  Maximum_Line_Length : NATURAL := 200; -- arbitrarily large for int buffers
package Report_Generator is
--| PURPOSE
--| Report_Generator supports the manipulation of a report file that is
--| supplemented by a table of contents file.  Both files are automatically
--| created when the Report_Generator comes into existence (it is up to
--| the user to remember to close them when done via the Close_Files method).
--| The user may then add lines to the report file with line wrapping and
--| use the other methods associated with the Report_Generator.
--|
--| The appearance of a page in the report file or table of contents file
--| is as follows:
--|                ---------------------------------
--|                | <Top_Margin blank lines>      |
--|                | <Page_Header_Line (centered)> |
--|                | <blank line>                  |
--|                | <first line of text body>     |
--|                |                               |
--|                |      ... <body> ...           |
--|                |                               |
--|                | <last line of text body>      |
--|                | <blank line>                  |
--|                | <page number (centered)>      |
--|                | <blank lines>                 |
--|                ---------------------------------
--|
--| The number of blank lines at the bottom of a page is the number
--| of total lines on the page minus the value of Bottom_Margin.  The
--| number of lines in the body of the page is the value of the
--| Bottom_Margin minus the value of the Top_Margin minus 4.
--|
--| AUTHOR
--| Richard Conn   1.0   26 Mar 96  Initial design

  procedure Create_Report_and_Contents_Files;
  --| PURPOSE
  --| Create and initialize the report file and the table of contents file.
  --|
  --| EXCEPTIONS
  --|   FILE_CREATION_ERROR - raised if one or both files cannot be created

  procedure Put_Line_to_Report_File
      (Line            : in STRING  := "";
       Center_the_Line : in BOOLEAN := FALSE;
       Indent_by       : in NATURAL := 0);
  --| PURPOSE
  --| Add a line to the report file.  If Indent_by is non-zero, reduce the
  --| Left_Margin setting for this line by this amount.  If Indent_by is
  --| greater than the Left_Margin, the Left_Margin will be set to zero.
  --| If Center_the_Line is TRUE, the line will be centered, if possible.
  --|
  --| EXCEPTIONS
  --|   FILE_WRITE_ERROR - raised if an error occurs, such as disk space
  --|                      filling up

  procedure Put_Line_to_Contents_File
      (Line               : in STRING  := "";
       Center_the_Line    : in BOOLEAN := FALSE;
       Append_Page_Number : in BOOLEAN := FALSE);
  --| PURPOSE
  --| Add a line to the table of contents file.  If Center_the_Line is TRUE,
  --| center the line on the page.  If Append_Page_Number is TRUE, place the
  --| page number of the current page in the report file onto the end of the
  --| line, right-justified.  If Append_Page_Number is TRUE, the
  --| Center_the_Line flag is ignored and the line is always left-justified.
  --|
  --| EXCEPTIONS
  --|   FILE_WRITE_ERROR - raised if an error occurs, such as disk space
  --|                      filling up

  procedure Eject_Page_in_Report_File;
  procedure Eject_Page_in_Contents_File;
  --| PURPOSE
  --| Force a page eject in either the report file or the table of contents file.
  --|
  --| EXCEPTIONS
  --|   FILE_WRITE_ERROR - raised if an error occurs, such as disk space
  --|                      filling up

  procedure Close_Report_and_Contents_Files;
  --| PURPOSE
  --| Close both the report file and the table of contents file.
  --|
  --| EXCEPTIONS
  --|   FILE_WRITE_ERROR - raised if an error occurs, such as disk space
  --|                      filling up

  FILE_CREATION_ERROR : exception;
  FILE_WRITE_ERROR    : exception;

end Report_Generator;
